/*****************************************************************************
 **
 ** Licensed Material - Property of IBM
 **
 ** 5724-D04
 **
 ** (C) Copyright IBM Corp. 2000 - 2004  All Rights Reserved
 **
 ** US Government Users Restricted Rights - Use, duplication or
 ** disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 **
 ***************************************************************************/
/**
* File 					:	MAB_Scanner.c
**/
#include <PalmOS.h>
#include "MAB_Scanner.h"
#include "MAB_ScannerMessages.h"

/***********************************************************************
 *
 * Function Name :		MAB_ScanSetAimDuration
 *
 * Description   : 		This is an exposed API function.
 *						It should be used to set the aim duration.
 *
 * Parameters	 :		duration: valid range is from 5 to 99. 
 *
 * Returns		 :    	Boolean : Returns true if the value is set, or else
 *						false.
 *
 * History		 :		Initial Creation 	May
 *
 ***********************************************************************/
Boolean MAB_ScanSetAimDuration(UInt16 duration){
	Boolean bret = true;
	if(duration >= 5 && duration <= 99)
		_MAB_SCAN_AIM_DURATION = duration;
	else
		bret = false;
	return bret;
}

/***********************************************************************
 *
 * Function Name :		MAB_ScanSetTriggeringMode
 *
 * Description   : 		This is an exposed API function.
 *						It will set the Triggering mode for the scanner.
 *
 * Parameters	 :		Mode: LEVEL or HOST or PULSE are the only allowed 
 * 						values
 *
 * Returns		 :    	Boolean : Returns true if the value is set, or else
 *						false.
 *
 * History		 :		Initial Creation 	May
 *
 ***********************************************************************/
Boolean MAB_ScanSetTriggeringMode(UInt16 mode){
	Boolean bret = true;
	if(mode == HOST || mode == LEVEL || mode == PULSE)
		_MAB_SCAN_TRIGGERING_MODE = mode;
	else
		bret = false;
	return bret;
}

/***********************************************************************
 *
 * Function Name :		MAB_ScanGetLastErrorCode
 *
 * Description   : 		This is an exposed API function.
 *						It will return the error code of the last function.
 *						called
 *
 * Parameters	 :		None.
 *
 * Returns		 :    	MAB_SCAN_STATUS: status.
 *
 * History		 :		Initial Creation 	May
 *
 ***********************************************************************/
MAB_SCAN_STATUS MAB_ScanGetLastErrorCode(){
	return _SCAN_STATUS;
}

/***********************************************************************
 *
 * Function Name :		MAB_ScanGetLastErrorMessage
 *
 * Description   : 		This is an exposed API function.
 *						It will return the last error message.
 *						
 * Parameters	 :		None
 *
 * Returns		 :    	CharPtr. The message.
 *
 * History		 :		Initial Creation 	May
 *
 ***********************************************************************/
CharPtr MAB_ScanGetLastErrorMessage(){
	return _SCAN_MESSAGE;
}

/***********************************************************************
 *
 * Function Name :		MAB_ScanDisableBarCodeType
 *
 * Description   : 		This is an exposed API function.
 *						This function should be used to disable any barcode 
 *						types from being scanned by the scanner. This function
 *						should be called before MAB_InitializeScanner to have a
 *						a meaningful effect. By default all 15 1D types are
 *						enabled.
 *						
 * Parameters	 :		barCodeType: The barcode type to be disable.
 *
 * Returns		 :    	None.
 *
 * History		 :		Initial Creation 	May
 *
 ***********************************************************************/
void MAB_ScanDisableBarCodeType(Word barCodeType){
	int i=0;
	for(i=0; i<COUNT_SUPPORTED_BARCODES; i++){
		if(supportedBarcodes[i] == barCodeType){
			isBarCodeEnabled[i] = false;
			break;
		}
	}
}

/***********************************************************************
 *
 * Function Name :		MAB_ScanEnableBarCodeType
 *
 * Description   : 		This is an exposed API function.
 *						This function should be used to enable any barcode 
 *						types from being scanned by the scanner. This function
 *						should be called before MAB_InitializeScanner to have a
 *						a meaningful effect. By default all 15 1D types are
 *						enabled.
 *						
 * Parameters	 :		barCodeType: The barcode type to be disable.
 *
 * Returns		 :    	None.
 *
 * History		 :		Initial Creation 	May
 *
 ***********************************************************************/
void MAB_ScanEnableBarCodeType(Word barCodeType){
	int i=0;
	for(i=0; i<COUNT_SUPPORTED_BARCODES; i++){
		if(supportedBarcodes[i] == barCodeType){
			isBarCodeEnabled[i] = true;
			break;
		}
	}
}

/***********************************************************************
 *
 * Function Name :		MAB_InitializeScanner
 *
 * Description   : 		This is an exposed API function.
 *						This function initializes the scanner hardware for 
 *						for a scan. 
 *						
 * Parameters	 :		isHardScan : TRUE if a initialization for hard scan is required.
 *									 FALSE if initialization for soft scan is required.
 *
 * Returns		 :    	MAB_SCAN_STATUS : status.
 *
 * History		 :		Initial Creation 	May
 *
 ***********************************************************************/
MAB_SCAN_STATUS MAB_InitializeScanner(){
	_SCAN_STATUS = ScanOpenDecoder();
	if(AssertScanStatus(_SCAN_STATUS == MAB_SCAN_STATUS_OK,__LINE__)){
		_SCAN_STATUS = ScanCmdScanEnable(); 
		AssertScanStatus(_SCAN_STATUS == MAB_SCAN_STATUS_OK,__LINE__);
		_SCAN_STATUS = InitForHardScan();
		if(AssertScanStatus(_SCAN_STATUS == MAB_SCAN_STATUS_OK,__LINE__))
		_SCAN_STATUS = ScanCmdSendParams(One_Short_High); // We hardcode here that a beep will be generated once all the parameters are submitted.
		AssertScanStatus(_SCAN_STATUS == MAB_SCAN_STATUS_OK,__LINE__);
	}
	return _SCAN_STATUS;
}

/***********************************************************************
 *
 * Function Name :		InitForHardScan
 *
 * Description   : 		This function initializes the scanner hardware for 
 *						a hardware or software triggered scan. 
 *						
 * Parameters	 :		None.
 *
 * Returns		 :    	MAB_SCAN_STATUS : status
 *
 * History		 :		Initial Creation 	May
 *
 ***********************************************************************/
MAB_SCAN_STATUS InitForHardScan(){
	UInt16 i;
	_SCAN_STATUS = ScanSetTriggeringModes(_HARDSCAN.triggeringMode);
	AssertScanStatus(_SCAN_STATUS == MAB_SCAN_STATUS_OK,__LINE__);
	
	for(i=0; i<COUNT_SUPPORTED_BARCODES; i++){
		_SCAN_STATUS = ScanSetBarcodeEnabled(_HARDSCAN.barCodeTypes[i],_HARDSCAN.barCodesToEnable[i]);
		AssertScanStatus(_SCAN_STATUS == MAB_SCAN_STATUS_OK,__LINE__);
	}
	ScanSetLaserOnTime(_HARDSCAN.aimDuration);
	
	return _SCAN_STATUS;
}

/***********************************************************************
 *
 * Function Name :		MAB_DeInitScanner
 *
 * Description   : 		This is an exposed API function.
 *						This function is used to uninitialize the scanner
 *						hardware.
 *						
 * Parameters	 :		None.
 *
 * Returns		 :    	MAB_SCAN_STATUS : The status.
 *						
 * History		 :		Initial Creation 	May
 *
 ***********************************************************************/

MAB_SCAN_STATUS MAB_DeInitScanner(void){
	_SCAN_STATUS = ScanCmdScanDisable();
	AssertScanStatus(_SCAN_STATUS == MAB_SCAN_STATUS_OK,__LINE__);
	_SCAN_STATUS = ScanCloseDecoder();
	AssertScanStatus(_SCAN_STATUS == MAB_SCAN_STATUS_OK,__LINE__);
	return _SCAN_STATUS;
}

/***********************************************************************
 *
 * Function Name :		DecodeBarCode
 *
 * Description   : 		This function actually calls the scanner API's to
 *						decode the scan. It is a function that is private
 *						to this module no to be exposed outside.
 *						
 * Parameters	 :		None.
 *
 * Returns		 :    	CharPtr : Returns the value just decoded. If a successful Decode
 *						did not take place then it returns a 'NR' as recd from the
 *						scanner API.
 *
 * History		 :		Initial Creation 	May
 *
 ***********************************************************************/
CharPtr DecodeBarCode(void){
	MESSAGE decodedBarCode;
	CharPtr pszData = NULL;
	if(AssertScanStatus((_SCAN_STATUS = ScanGetDecodedData(&decodedBarCode)) == MAB_SCAN_STATUS_OK,__LINE__)){
		pszData = (CharPtr)decodedBarCode.data;
	}
	return pszData;
}

/***********************************************************************
 *
 * Function Name :		MAB_DecodeScan
 *
 * Description   : 		This is an exposed API function.
 *						This function will be called to decode the scan that 
 *						just happened. This should be called from the scanDecodeEvent 
 *						handler.
 *						
 * Parameters	 :		None.
 *
 * Returns		 :    	CharPtr : Returns the value just decoded. If a successful Decode
 *						did not take place then it returns a null.
 *
 * History		 :		Initial Creation 	May
 *
 ***********************************************************************/
CharPtr MAB_DecodeScan (void){
   CharPtr pszDecodedBarCode = NULL;
	pszDecodedBarCode = DecodeBarCode();
	if(pszDecodedBarCode != NULL & (pszDecodedBarCode[0]=='N' && pszDecodedBarCode[1]=='R'))
		pszDecodedBarCode = NULL;
	return pszDecodedBarCode;
}


/***********************************************************************
 *
 * Function Name :		MAB_DoSoftScan
 *
 * Description   : 		This is an exposed API function.
 *						This function should be called when the scanner 
 *						should be triggered from a button or a menu item.
 *						This is called as soft-scan.
 *						
 * Parameters	 :		None.
 *
 * Returns		 :    	
 *						
 *
 * History		 :		Initial Creation 	May
 *
 ***********************************************************************/
MAB_SCAN_STATUS MAB_DoSoftScan(){
	_SCAN_STATUS = ScanCmdStartDecode();
	AssertScanStatus(_SCAN_STATUS == MAB_SCAN_STATUS_OK,__LINE__);
	return _SCAN_STATUS;
}

/***********************************************************************
 *
 * Function Name :		MAB_ScanHandleNoDecode
 *
 * Description   : 		This is an exposed API function.
 *						
 *
 * Parameters	 :		None.
 *
 * Returns		 :    	None.
 *
 * History		 :		Initial Creation 	May
 *
 ***********************************************************************/
void MAB_ScanHandleNoDecode(){
	MAB_ScanErrorHandler(MAB_SCAN_NODECODE);
}


/***********************************************************************
 *
 * Function Name :		MAB_ScanErrorHandler
 *
 * Description   : 		This is an exposed API function.
 *						This functions handles all unsuccessful conditions. 
 *						It will fetch the corresponding message from pabmessages.h file
 *						and store it in gDefaultScanErrorHandlerData.szErrorMsg
 *						
 *
 * Parameters	 :		status : The error code
 *
 * Returns		 :    	boolean: Whether the calling function should continue or not.
 *
 * History		 :		Initial Creation 	May
 *
 ***********************************************************************/
Boolean MAB_ScanErrorHandler(UInt16 status){
	Boolean bshouldContinue = true;
	CharPtr pszMessage = NULL;
	_SCAN_STATUS = status;
	switch(status){
		case MAB_SCAN_BARCODE_NOT_SUPPORTED:
			pszMessage = (CharPtr)MemPtrNew(StrLen(MAB_SCAN_BARCODE_NOT_SUPPORTED_MESSAGE) + 1);
			StrCopy(pszMessage,MAB_SCAN_BARCODE_NOT_SUPPORTED_MESSAGE);
			break;
			
		case MAB_SCAN_COMMUNICATIONS_ERROR:
			pszMessage = (CharPtr)MemPtrNew(StrLen(MAB_SCAN_COMMUNICATIONS_ERROR_MESSAGE) + 1);
			StrCopy(pszMessage,MAB_SCAN_COMMUNICATIONS_ERROR_MESSAGE);
			break;
			 
		case MAB_SCAN_BAD_PARAM:
			pszMessage = (CharPtr)MemPtrNew(StrLen(MAB_SCAN_BAD_PARAM_MESSAGE) + 1);
			StrCopy(pszMessage,MAB_SCAN_BAD_PARAM_MESSAGE);
			break;
			
		case MAB_SCAN_BATCH_ERROR:
			pszMessage = (CharPtr)MemPtrNew(StrLen(MAB_SCAN_BATCH_ERROR_MESSAGE) + 1);
			StrCopy(pszMessage,MAB_SCAN_BATCH_ERROR_MESSAGE);
			break;
			
		case MAB_SCAN_NODECODE:
			pszMessage = (CharPtr)MemPtrNew(StrLen(MAB_SCAN_NODECODE_MESSAGE) + 1);
			StrCopy(pszMessage,MAB_SCAN_NODECODE_MESSAGE);
			break;
		
		case MAB_SCAN_SOFTSCAN_FAILED:
			pszMessage = (CharPtr)MemPtrNew(StrLen(MAB_SCAN_SOFTSCAN_FAILED_MESSAGE) + 1);
			StrCopy(pszMessage,MAB_SCAN_SOFTSCAN_FAILED_MESSAGE);
			break;
		
		case MAB_SCAN_CURRENT_FIELD_NOT_SCANAWARE:
			pszMessage = (CharPtr)MemPtrNew(StrLen(MAB_SCAN_CURRENT_FIELD_NOT_SCANAWARE_MESSAGE) + 1);
			StrCopy(pszMessage,MAB_SCAN_CURRENT_FIELD_NOT_SCANAWARE_MESSAGE);
			break;
		
		case MAB_SCAN_UNKNOWN_ERROR:
		default:
			pszMessage = (CharPtr)MemPtrNew(StrLen(MAB_SCAN_UNKNOWN_ERROR_MESSAGE) + 1);
			StrCopy(pszMessage,MAB_SCAN_UNKNOWN_ERROR_MESSAGE);
			break;
	}
	if(gDefaultScanErrorHandlerData.szErrorMsg){
		MemPtrFree(gDefaultScanErrorHandlerData.szErrorMsg);
	}
	gDefaultScanErrorHandlerData.szErrorMsg = NULL;
	gDefaultScanErrorHandlerData.szErrorMsg = (CharPtr)MemPtrNew(StrLen(pszMessage) + 1);
	StrCopy(gDefaultScanErrorHandlerData.szErrorMsg,pszMessage);
	if(gDefaultScanErrorHandlerData.alertID != -1)
		FrmCustomAlert(gDefaultScanErrorHandlerData.alertID,pszMessage,"","");

	if(pszMessage)
		MemPtrFree(pszMessage);
	return true; //for now this function ony return true.
}

/***********************************************************************
 *
 * Function Name :		MAB_ScanIsFocusOnScanAwareField
 *
 * Description   : 		Finds the field which has the caret postion. if the
 *						caret is in a field which is scan aware then return
 *						the field id of that field.   
 *						
 * Parameters	 :		Array of fields that are scan aware.
 *
 * Returns		 :    	Word : Returns the FieldID of the field which has the 
 *						current caret position 
 *								-1 if the current control is not scan aware.
 *
 * History		 :		Initial Creation 	May
 *
 ***********************************************************************/
Word MAB_ScanIsFocusOnScanAwareField(FormPtr pFrm, Word scanAwareFields[], UInt16 nCount){
	int i=0;
	FieldPtr pfield = NULL;
	Word fieldInFocus = -1;
	FieldAttrType FieldAttr;
	
	if(nCount == 1)
		return scanAwareFields[0];
	
	for(i=0; i<nCount; i++){
		pfield = (FieldPtr)FrmGetObjectPtr(pFrm,FrmGetObjectIndex(pFrm,scanAwareFields[i]));
		if(pfield != NULL){
			FldGetAttributes(pfield,&FieldAttr);
			if(&FieldAttr != 0 & FieldAttr.hasFocus) {
				fieldInFocus = scanAwareFields[i];
				break;
			}
		}
	}
	
	if(fieldInFocus == -1)
		MAB_ScanErrorHandler(MAB_SCAN_CURRENT_FIELD_NOT_SCANAWARE);

	return fieldInFocus;
}

/***********************************************************************
 *
 * Function Name :		MAB_ScanGetNextScanAwareField
 *
 * Description   : 		Sets the focus on the next scan aware field selected
 *						by the user.
 *						
 * Parameters	 :		Last scan aware field in focus.
 *						Array of fields that are scan aware.
 *						
 *
 * Returns		 :    	Word : Next field that should have focus.  
 *							   
 *
 * History		 :		Initial Creation 	May
 *
 ***********************************************************************/
Word MAB_ScanGetNextScanAwareField(FormPtr pFrm, Word lastField, Word scanAwareFields[], UInt16 nCount){
	int i=0;
	Word fieldInFocus = scanAwareFields[nCount - 1];
	
	for(i=0; i<nCount; i++){
		if(scanAwareFields[i] == lastField && i != nCount-1){
			fieldInFocus = scanAwareFields[i+1];
			break;
		}
	}
	
	return fieldInFocus;
}

/***********************************************************************
 *
 * Function Name :		MAB_ScanSetAlertIDForMessages
 *
 * Description   : 		This is an exposed API function.
 *						
 *						
 * Parameters	 :		AlertID
 *						
 *
 * Returns		 :    	None
 *
 * History		 :		Initial Creation 	May
 *
 ***********************************************************************/
void MAB_ScanSetAlertIDForMessages(Word alertID){
	if(gDefaultScanErrorHandlerData.alertID == -1)
		gDefaultScanErrorHandlerData.alertID = alertID;	
}

/***********************************************************************
 *
 * Function Name :		AssertScanStatus
 *
 * Description   : 		This function is purely to generate debug messages.
 *						This function will compile to produce a non-redundant 
 *						functionality only when the code is compiled in debug mode.
 *						
 * Parameters	 :		bAssert : Is the Assertion true or false.
 *						nLineNo	: Line number which cause this assertion failure.
 *
 * Returns		 :    	boolean: returns back the bAssert.
 *
 * History		 :		Initial Creation 	May
 *
 ***********************************************************************/

Boolean AssertScanStatus(Boolean bAssert,UInt16 nLineNo){
//#if __DEBUG_MODE__
	if(bAssert == false)
		MAB_ScanErrorHandler(gDefaultScanErrorHandlerData.currentStatus);
//#endif
	return bAssert;
}
